package com.ruoyi.web.controller.api;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.TPatient;
import com.ruoyi.system.domain.TPatientHospitalDoctor;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.AppPatientRecord;
import com.ruoyi.system.domain.req.AppletPatientHospitalDoctor;
import com.ruoyi.system.domain.resp.PatientHistoryRecordResp;
import com.ruoyi.system.service.ITPatientHospitalDoctorService;
import com.ruoyi.system.service.ITPatientService;
import com.ruoyi.system.service.ITRecordHistoryService;
import com.ruoyi.system.service.ITRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Api("mp接口")
@RestController
@RequestMapping("/api/mp")
public class MpController extends BaseController {
    @Autowired
    private ITRecordService tRecordService;

    @Autowired
    private ITRecordHistoryService recordHistoryService;

    @Resource
    private ITPatientHospitalDoctorService itPatientHospitalDoctorService;

    @Resource
    private ITPatientService patientService;


    @ApiOperation("患者诊断记录")
    @GetMapping("/patient/history/record")
    @Log(title = "获取患者诊断记录", businessType = BusinessType.OTHER)
    public AjaxResult queryPatientHistoryRecord()
    {
        TRecord tRecord = new TRecord();
        tRecord.setPatientId(getUserId());
        List<PatientHistoryRecordResp> resp = tRecordService.queryPatientHistoryRecord(tRecord);
        return AjaxResult.success(resp);
    }

    /**
     * 重新安排
     *
     * @return {@code AjaxResult}
     */
    @ApiOperation("APP重新拍摄")
    @Log(title = "APP重新拍摄", businessType = BusinessType.INSERT)
    @GetMapping("/record/reschedule/{recordId}")
    public AjaxResult reschedule(@PathVariable("recordId") Long recordId){
        // 逻辑删除本条记录
        TRecord tRecord = tRecordService.selectTRecordById(recordId);

        //备份历史记录
        recordHistoryService.saveHistory(recordId);

        tRecord.setId(recordId);
        tRecord.setUpdateResult("");
        tRecord.setUpdateResult2("");
        tRecordService.updateTRecord(tRecord);
        AjaxResult success = AjaxResult.success();
        success.put("recordId", tRecord.getId());
        return success;
    }

    /**
     * 病人预约
     *
     * @param appPatientHospitalDoctor 应用病人医院医生
     * @return {@code AjaxResult}
     */
    @ApiOperation("APP患者预约")
    @Log(title = "APP患者预约", businessType = BusinessType.INSERT)
    @GetMapping("/patient/makeAnAppointment")
    public AjaxResult patientsToMakeAnAppointment(AppletPatientHospitalDoctor appPatientHospitalDoctor)
    {
        // 查询该患者当天是否有预约记录
        TRecord tRecord = new TRecord();
        tRecord.setPatientId(appPatientHospitalDoctor.getPatientId());
        tRecord.setHospitalId(appPatientHospitalDoctor.getHospitalId());
        tRecord.setDoctorId(appPatientHospitalDoctor.getDoctorId());
        tRecord.setAppointmentTime(DateUtils.dateTime(new Date()));
        if (whetherRegistration(tRecord, "0") || whetherRegistration(tRecord, "2") || whetherRegistration(tRecord, "3")) {
            AjaxResult ajaxResult = AjaxResult.error();
            ajaxResult.put("msg", "当前患者已登记过信息！");
            return ajaxResult;
        }

        // 患者预约，新增t_record中的一条记录;判断关联表是否存在当前关系
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        BeanUtils.copyProperties(appPatientHospitalDoctor, tPatientHospitalDoctor);
        tRecord.setCreateBy(getUsername());
        tRecord.setStatus("0");
        tRecordService.insertTRecord(tRecord);
        List<TPatientHospitalDoctor> tPHDs = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPHDs.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }
        TRecord queryRecord = new TRecord();
        queryRecord.setId(tRecord.getId());
        List<PatientHistoryRecordResp> resp = tRecordService.queryPatientHistoryRecord(queryRecord);
        AjaxResult ajaxResult = AjaxResult.success();
        ajaxResult.put("record", CollectionUtils.isEmpty(resp) ? null : resp.get(0));
        ajaxResult.put("msg", "登记成功");
        return ajaxResult;
    }

    private Boolean whetherRegistration(TRecord tRecord, String status) {
        tRecord.setStatus(status);
        List<TRecord> tRecords = tRecordService.selectTRecordList(tRecord);
        return !CollectionUtils.isEmpty(tRecords);
    }

    @ApiOperation("APP查询记录状态")
    @Log(title = "APP查询记录状态", businessType = BusinessType.OTHER)
    @GetMapping("/record/status")
    public AjaxResult queryRecordState(AppPatientRecord appPatientHospitalDoctor){
        AjaxResult ajaxResult = AjaxResult.success();
        TPatient tPatient = new TPatient();
        tPatient.setName(appPatientHospitalDoctor.getName());
        tPatient.setPhone(appPatientHospitalDoctor.getPhone());
        List<TPatient> tPatients = patientService.selectTPatientList(tPatient);
        if (CollectionUtils.isEmpty(tPatients)) {
            ajaxResult.put("status", null);
        }
        // 查询该患者当天是否有预约记录
        TRecord tRecord = new TRecord();
        tRecord.setPatientId(tPatients.get(0).getId());
        tRecord.setHospitalId(appPatientHospitalDoctor.getHospitalId());
        tRecord.setDoctorId(appPatientHospitalDoctor.getDoctorId());
        tRecord.setAppointmentTime(DateUtils.dateTime(new Date()));
        List<TRecord> tRecords = tRecordService.selectTRecordList(tRecord);
        tRecords = tRecords.stream()
                .sorted(Comparator.comparing(TRecord::getUploadTime, Comparator.nullsLast(Date::compareTo)).reversed())
                .collect(Collectors.toList());
        ajaxResult.put("status", tRecords.get(0).getStatus());
        return ajaxResult;
    }
}
