package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.TPatientHospitalDoctor;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.TRecordResultReq;
import com.ruoyi.system.domain.resp.PatientHistoryRecordResp;
import com.ruoyi.system.domain.resp.RecordHistoryDataResp;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.mapper.TRecordMapper;
import com.ruoyi.system.service.ITRecordService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 诊断记录信息Service业务层处理
 *
 * @author ruoyi
 * @date 2022-08-03
 */
@Service
public class TRecordServiceImpl implements ITRecordService
{
    private static final Logger logger = LoggerFactory.getLogger(TRecordServiceImpl.class);

    private static Map<String, JSONObject> jsonKeyMap = new HashMap<>();

    private static Map<String, String> newJsonKeyMap = new HashMap<String, String>(){{
        put("sx_vein_color", "sx_vein_color2");
        put("sz_color", "sz_color2");
        put("st_color", "st_color2");
    }};

    /**
     * AI返回结果中可能为空值的key
     */
    private static final List<String> nullValKeys = new ArrayList<String>(){{
        add("st_avg_yellow");
        add("sz_avg_purple");
        add("sz_avg_red");
        add("sx_vein_avg_purple");
    }};

    /**
     * 字段对应v的值为图片base64的key
     */
    private static final List<String> ignoreKeys = new ArrayList<String>(){{
        add("img_sz");
        add("img_sz_yudian");
        add("img_sz_yuban");
        add("img_sz_liewen");
        add("img_st");
        add("img_sx");
        add("img_sx_vein");
    }};

    /**
     * 需要获取量化值的key
     */
    private static final List<String> getQuantificationKeys = new ArrayList<String>(){{
        add("sx_vein_width");
        add("sx_vein_length");
    }};

    @Autowired
    private TRecordMapper tRecordMapper;

    /**
     * 查询诊断记录信息
     *
     * @param id 诊断记录信息主键
     * @return 诊断记录信息
     */
    @Override
    public TRecord selectTRecordById(Long id)
    {
        return tRecordMapper.selectTRecordById(id);
    }

    /**
     * 查询诊断记录信息列表
     *
     * @param tRecord 诊断记录信息
     * @return 诊断记录信息
     */
    @Override
    public List<TRecord> selectTRecordList(TRecord tRecord)
    {
        return tRecordMapper.selectTRecordList(tRecord);
    }

    /**
     * 新增诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int insertTRecord(TRecord tRecord)
    {
        if (null == tRecord.getCreateTime()) {
            tRecord.setCreateTime(DateUtils.getNowDate());
        }
        return tRecordMapper.insertTRecord(tRecord);
    }

    /**
     * 修改诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int updateTRecord(TRecord tRecord)
    {
        tRecord.setUpdateTime(DateUtils.getNowDate());
        return tRecordMapper.updateTRecord(tRecord);
    }

    /**
     * 批量删除诊断记录信息
     *
     * @param ids 需要删除的诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordByIds(Long[] ids)
    {
        return tRecordMapper.deleteTRecordByIds(ids);
    }

    /**
     * 删除诊断记录信息信息
     *
     * @param id 诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordById(Long id)
    {
        return tRecordMapper.deleteTRecordById(id);
    }

    @Override
    public List<TRecordResp> selectTRecordListWithoutResult(TRecordResultReq tRecord)
    {
        return tRecordMapper.selectTRecordListWithoutResult(tRecord);
    }

    @Override
    public List<TRecordResp> queryHomePatientList(TRecordResultReq tRecord) {
        List<TRecordResp> tRecordResps = tRecordMapper.selectHomePatientList(tRecord);
        return tRecordResps;
    }

    @Override
    public TRecordResp selectById(Long id)
    {
        TRecordResp tRecordResp = tRecordMapper.selectById(id);
        if (null == tRecordResp.getAiResult()) {
            return tRecordResp;
        }
        String aiResult = tRecordResp.getAiResult();
        String updateResult = tRecordResp.getUpdateResult();
        if (!aiResult.contains("sz_color2")) {
            tRecordResp.setAiResult(dealJsonString(aiResult));
            tRecordResp.setUpdateResult(dealJsonString(updateResult));
        }
        return tRecordResp;
    }

    /**
     * 处理json字符串
     *
     * @param jsonString json字符串
     * @return {@code String}
     */
    private String dealJsonString(String jsonString) {
        if (null == jsonString || "".equals(jsonString)) {
            return "";
        }
        // 将老数据结构对应的数据读出来
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        readJsonToMap(jsonObject.getJSONObject("data"));
        // 加载新数据结构模板
        String localPath = RuoYiConfig.getJSONTemplatePath();
        String jsonStr = com.ruoyi.common.utils.file.FileUtils.readJsonFile(localPath +"/" + "Result.json");
        JSONObject newJsonObj = JSONObject.parseObject(jsonStr);
        // 构建数据
        newJsonObj.put("data", buildJson(newJsonObj.getJSONObject("data")));
        return newJsonObj.toJSONString();
    }

    /**
     * 读取json，将对应的k v存入map
     *
     * @param obj obj
     */
    private void readJsonToMap(Object obj) {
        if (obj instanceof JSONObject) {
            for (Map.Entry<String, Object> entry : ((JSONObject) obj).entrySet()) {
                if ((entry.getValue() instanceof JSONObject)) {
                    // 子对象中包含v, qu..
                    if (null != ((JSONObject) entry.getValue()).get("anomaly")) {
                        // json to map
                        jsonKeyMap.put(entry.getKey(), (JSONObject) entry.getValue());
                        continue;
                    }
                    // 递归调用
                    readJsonToMap(entry.getValue());
                }
            }
        }
    }

    /**
     * 构建新结构的json
     *
     * @param obj obj
     * @return {@code JSONObject}
     */
    private JSONObject buildJson(Object obj) {
        JSONObject jsonResult = null;
        if (obj instanceof JSONObject) {
            jsonResult = (JSONObject) obj;
            for (Map.Entry<String, Object> entry : ((JSONObject) obj).entrySet()) {
                if ((entry.getValue() instanceof JSONObject)) {
                    // 子对象中包含v, qu..
                    if (null != ((JSONObject) entry.getValue()).get("anomaly")) {
                        // build json
                        String key = entry.getKey();
                        String newKey = newJsonKeyMap.get(key);
                        if (null == newKey) {
                            newKey = key;
                        }
                        JSONObject jsonValByKey = jsonKeyMap.get(newKey);
                        if (null != jsonValByKey) {
                            jsonResult.put(key, jsonValByKey);
                        }
                        continue;
                    }
                    // 递归调用
                    JSONObject jsonChild = buildJson(entry.getValue());
                    jsonResult.put(entry.getKey(), jsonChild);
                }
            }
        }
        return jsonResult;
    }

    @Override
    public List<TRecordResp> queryRecordListByPHDids(TPatientHospitalDoctor tPatientHospitalDoctor)
    {
        List<TRecordResp> tRecordResps = tRecordMapper.queryRecordListByPHDids(tPatientHospitalDoctor);
        TRecordResp tRecordResp1 = tRecordResps.get(0);
        if (null != tRecordResp1 && null != tRecordResp1.getAiResult() && !tRecordResp1.getAiResult().contains("sz_color2")) {
            tRecordResps = tRecordResps.stream().peek(tRecordResp -> {
                if (null != tRecordResp.getAiResult()) {
                    tRecordResp.setAiResult(dealJsonString(tRecordResp.getAiResult()));
                }
                if (null != tRecordResp.getUpdateResult()) {
                    tRecordResp.setUpdateResult(dealJsonString(tRecordResp.getUpdateResult()));
                }
            }).collect(Collectors.toList());
        }
        return tRecordResps;
    }

    @Override
    public AjaxResult getHistoryList(TRecord tRecord)
    {
        logger.info("get history list param is : {} ", tRecord);
        List<TRecord> tRecords = tRecordMapper.selectTRecordList(tRecord);
        tRecords.removeIf(o->"0".equals(o.getStatus()));
        if (CollectionUtils.isEmpty(tRecords)){
            return AjaxResult.success();
        }
        List<TRecord> records = tRecords.stream().sorted(Comparator.comparing(TRecord::getCreateTime)).collect(Collectors.toList());
        RecordHistoryDataResp recordHistoryDataRespMap = getDataList(records);
        return AjaxResult.success(recordHistoryDataRespMap);
    }

    private RecordHistoryDataResp getDataList(List<TRecord> records)
    {
        RecordHistoryDataResp recordHistoryDataResp = new RecordHistoryDataResp();
        // 构造时间轴坐标值
        List<Date> dateList = records.stream().map(TRecord -> {
            if (null != TRecord.getUploadTime()) {
                return TRecord.getUploadTime();
            } else {
                return TRecord.getCreateTime();
            }
        }).collect(Collectors.toList());
        recordHistoryDataResp.setDateList(dateList);
        // 构造纵坐标值
        List<JSONObject> result = records.stream().map(tRecord -> {
            JSONObject jsonObject = new JSONObject();
            if(StringUtils.isEmpty(tRecord.getUpdateResult())){
                jsonObject = JSONObject.parseObject(tRecord.getAiResult());
            }else if(!StringUtils.isEmpty(tRecord.getUpdateResult())){
                jsonObject = JSONObject.parseObject(tRecord.getUpdateResult());
            }
            return jsonObject;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(result)){
            HashMap<String, ArrayList<Object>> map = new HashMap<>();
            result.forEach(jsonObject -> {
                getJsonData(jsonObject.getJSONObject("data"), "data", map, ignoreKeys);
            });
            recordHistoryDataResp.setHistoryDataMap(map);
        }
        return recordHistoryDataResp;
    }

    /**
     * 获取json数据
     *
     * @param obj            obj
     * @param listName       列表名称
     * @param historyDataMap 历史数据
     * @param ignoreKeyList  忽略键列表
     */
    private void getJsonData(Object obj, String listName, HashMap<String, ArrayList<Object>> historyDataMap, List<String> ignoreKeyList) {
        // aiResult结果集中只包括JSONObject,不包括JSONArray
        if (obj instanceof JSONObject) {
            // "sx_vein_avg_purple": {}
            // "sx_vein_avg_purple": {"anomaly": 0}
            // 这两种情况应该添加默认值
            if (nullValKeys.contains(listName) && null == ((JSONObject) obj).get("v")) {
                // 加默认值
                doSetHistoryData(historyDataMap, listName, "0");
            }
            for (Map.Entry<String, Object> entry : ((JSONObject) obj).entrySet()) {
                // 忽略带图片字段
                if (ignoreKeyList.contains(listName)) {
                    continue;
                }
                if ((entry.getValue() instanceof JSONObject)) {
                    // 递归调用
                    getJsonData(entry.getValue(), entry.getKey(), historyDataMap, ignoreKeyList);
                } else {
                    // sx_vein_length和sx_vein_width，使用quantification量化字段
                    if (getQuantificationKeys.contains(listName) && "quantification".equals(entry.getKey())) {
                        doSetHistoryData(historyDataMap, listName, entry.getValue());
                    }
                    //sz_avg_purple: { "anomaly": 0, "v": "正常" }
                    // anomaly属于预留字段，暂时无用; 各个任务中的字段v是任务实际值
                    if (!getQuantificationKeys.contains(listName) && "v".equals(entry.getKey())) {
                        doSetHistoryData(historyDataMap, listName, entry.getValue());
                    }
                }
            }
        }
    }

    /**
     * 做历史数据
     *
     * @param historyDataMap 历史数据map
     * @param listName       列表名称
     * @param val            值
     */
    private void doSetHistoryData(HashMap<String, ArrayList<Object>> historyDataMap, String listName, Object val) {
        ArrayList<Object> list;
        if (null == historyDataMap.get(listName)) {
            list = new ArrayList<>();
        } else {
            list = historyDataMap.get(listName);
        }
        list.add(val);
        historyDataMap.put(listName, list);
    }

    /**
     * 查询医生诊断次数
     *
     * @param doctorId 医生id
     * @return int
     */
    @Override
    public int queryDoctorNumberOfDiagnosis(Long doctorId) {
        return tRecordMapper.queryDoctorNumberOfDiagnosis(doctorId);
    }

    @Override
    public List<PatientHistoryRecordResp> queryPatientHistoryRecord(TRecord tRecord) {
        return tRecordMapper.queryPatientHistoryRecord(tRecord);
    }

    /**
     * 查询今天记录
     *
     * @param tRecord t记录
     * @return {@code List<TRecordResp>}
     */
    @Override
    public List<TRecordResp> selectTodayRecord(TRecordResultReq tRecord) {
        return tRecordMapper.selectTodayRecord(tRecord);
    }


}
